require 'spec_helper'

describe "T002: Codeforces Driver" do
  before(:each) do
    @driver = Contest::Driver::CodeforcesDriver.new
    allow(@driver).to receive(:sleep).and_return(0)
  end

  context "A001: #get_status_wait" do
    before do
      # basic status_log
      WebMock.stub_request(
        :get,
        /http:\/\/codeforces.com\/contest\/[0-9A-Z]*\/my/
      ).to_return(
        :status => 200,
        :body => read_file('/mock/t002/my_submissions.html'),
        :headers => {
          'Content-Type' => 'text/html',
        },
      )
    end

    it "check status" do
      ret = @driver.send :get_status_wait, 11111, 22222
      expect(ret).to include "Accepted"
    end
  end

  context "A002: #submit" do
    before do
      FileUtils.touch 'test_source.rb'

      # basic status_log
      WebMock.stub_request(
        :get,
        /http:\/\/codeforces.com\/contest\/222222\/my/
      ).to_return(
        :status => 200,
        :body => read_file('/mock/t002/codeforces_wait_result.html'),
        :headers => {
          'Content-Type' => 'text/html',
        },
      )

      # basic status_log
      WebMock.stub_request(
        :get,
        /http:\/\/codeforces.com\/enter/
      ).to_return(
        :status => 200,
        :body => read_file('/mock/t002/codeforces_enter.html'),
        :headers => {
          'Content-Type' => 'text/html',
        },
      )

      # enter page
      WebMock.stub_request(
        :post,
        /http:\/\/codeforces.com\/enter/
      ).to_return(
        :status => 200,
        :headers => {
          'Content-Type' => 'text/html',
        },
      )

      # submit
      WebMock.stub_request(
        :get,
        /http:\/\/codeforces.com\/contest\/[0-9]*\/submit/
      ).to_return(
        :status => 200,
        :body => read_file('/mock/t002/codeforces_submit.html'),
        :headers => {
          'Content-Type' => 'text/html',
        },
      )

      # submit
      WebMock.stub_request(
        :post,
        /http:\/\/codeforces.com\/contest\/[0-9]*\/submit.*/
      ).to_return(
        :status => 302,
        :headers => {
          'Content-Type' => 'text/html',
          'Location' => 'http://codeforces.com/after_submit',
        },
      )

      # problemst status
      WebMock.stub_request(
        :get,
        /http:\/\/codeforces.com\/after_submit/
      ).to_return(
        :status => 200,
        :body => read_file('/mock/t002/codeforces_after_submit.html'),
        :headers => {
          'Content-Type' => 'text/html',
        },
      )
    end

    it "should return commit message" do
      @driver.config.merge!(
        "user" => "test_user",
        "password" => "password",
      )
      @driver.options.merge!(
        :contest_id => '222222',
        :problem_id => 'A',
        :source => 'test_source.rb',
      )
      expect(@driver.submit).to include "Codeforces 222222A: Accepted"
    end

    it "check events" do
      @flag_start         = false
      @flag_before_login  = false
      @flag_after_login   = false
      @flag_before_submit = false
      @flag_after_submit  = false
      @flag_before_wait   = false
      @flag_after_wait    = false
      @flag_finish        = false

      proc_start = Proc.new do
        @flag_start = true
      end
      proc_before_login = Proc.new do |info|
        @flag_before_login = true
      end
      proc_after_login = Proc.new do
        @flag_after_login = true
      end
      proc_before_submit = Proc.new do |info|
        @flag_before_submit = true
      end
      proc_after_submit = Proc.new do
        @flag_after_submit = true
      end
      proc_before_wait = Proc.new do
        @flag_before_wait = true
      end
      proc_after_wait = Proc.new do
        @flag_after_wait = true
      end
      proc_finish = Proc.new do
        @flag_finish = true
      end

      @driver.on 'start', proc_start
      @driver.on 'before_login', proc_before_login
      @driver.on 'after_login', proc_after_login
      @driver.on 'before_submit', proc_before_submit
      @driver.on 'after_submit', proc_after_submit
      @driver.on 'before_wait', proc_before_wait
      @driver.on 'after_wait', proc_after_wait
      @driver.on 'finish', proc_finish

      @driver.config.merge!(
        "user" => "test_user",
        "password" => "password",
      )
      @driver.options.merge!(
        :contest_id => '222222',
        :problem_id => 'A',
        :source => 'test_source.rb',
      )

      @driver.submit

      @flag = @flag_start && @flag_before_login && @flag_after_login && @flag_before_submit && @flag_after_submit && @flag_before_wait && @flag_after_wait && @flag_finish
      expect(@flag).to eq true
    end
  end
end

